
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '1.26.13',
    'branch_nick': 'release-1.26-LTS',
    'revision_id': '7559d7c09c43e3f1b5024eb37a95bad4352f11be',
    'revno': 10947
}
